' (C) Copyright 2011 by Autodesk, Inc. 
'
' Permission to use, copy, modify, and distribute this software
' in object code form for any purpose and without fee is hereby
' granted, provided that the above copyright notice appears in
' all copies and that both that copyright notice and the limited
' warranty and restricted rights notice below appear in all
' supporting documentation.
'
' AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
' AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
' MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK,
' INC. DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL
' BE UNINTERRUPTED OR ERROR FREE.
'
' Use, duplication, or disclosure by the U.S. Government is
' subject to restrictions set forth in FAR 52.227-19 (Commercial
' Computer Software - Restricted Rights) and DFAR 252.227-7013(c)
' (1)(ii)(Rights in Technical Data and Computer Software), as
' applicable.
'
Imports System
Imports System.Collections.Generic
Imports System.Linq

Imports Autodesk.Revit.DB
Imports Autodesk.Revit.DB.Architecture
Imports Autodesk.Revit.UI
Imports Autodesk.Revit.UI.Selection
Imports Autodesk.Revit.ApplicationServices
Imports Autodesk.Revit.Attributes

' Lab5 can duplicate a specified group to the position user picked.
' * It shows the usage of following basic APIs 
' * 
' * Filter to select specified elements
' * Try-Catch usage.
' *  
' * Before run this command, a group should  already be in the active
' * document.
' 


<TransactionAttribute(TransactionMode.Manual)> _
<RegenerationAttribute(RegenerationOption.Manual)> _
Public Class Lab5PickFilter
  Implements IExternalCommand
  Public Function Execute(ByVal commandData As ExternalCommandData, _
                          ByRef message As String, _
                          ByVal elements As ElementSet) As Result _
                          Implements IExternalCommand.Execute
    Dim uiApp As UIApplication = commandData.Application
    Dim doc As Document = uiApp.ActiveUIDocument.Document

    Try
      'Define a Reference object to accept the pick result.
      Dim pickedRef As Reference = Nothing

      'Pick a group using a filter.
      'So only group can be selected.
      Dim sel As Selection = uiApp.ActiveUIDocument.Selection
      Dim selFilter As New GroupPickFilter()
      pickedRef = sel.PickObject( _
      ObjectType.Element, selFilter, "Please select a group")

      'Get picked Element
      'In Revit 2011 we can get element directly by this line
            'Dim elem As Element = pickedRef.Element
      'In Revit 2012, Reference.Element property is obsolete. 
      'The picked element can be retrieved by this:
            Dim elem As Element = doc.GetElement(pickedRef)

      Dim group As Group = TryCast(elem, Group)

      'pick a point
      Dim point As XYZ = sel.PickPoint("Please pick a point")

      'place the group
      Dim trans As New Transaction(doc)
      trans.Start("Lab")
      doc.Create.PlaceGroup(point, group.GroupType)
      trans.Commit()

      'If user right click or press ESC button, handle the exception
    Catch generatedExceptionName As  _
            Autodesk.Revit.Exceptions.OperationCanceledException
      Return Result.Cancelled
      'Catch other error.
    Catch ex As Exception
      message = ex.Message
      Return Result.Failed
    End Try

    Return Result.Succeeded
  End Function
End Class


''' <summary>
''' Filter to constrain picking to model groups. Only model groups 
''' are highlighted and can be selected when cursor is hovering.
''' </summary>
Public Class GroupPickFilter
  Implements ISelectionFilter
  Public Function AllowElement(ByVal e As Element) _
  As Boolean Implements ISelectionFilter.AllowElement
    Return (e.Category.Id.IntegerValue.Equals( _
            CInt(BuiltInCategory.OST_IOSModelGroups)))
  End Function
  Public Function AllowReference( _
  ByVal r As Reference, ByVal p As XYZ) _
  As Boolean Implements ISelectionFilter.AllowReference
    Return False
  End Function
End Class

